/* oscdraw.c
 * 
 * Licensed under WTFPL
 * (c) 2010 LexsZero <lexszero@gmail.com>
 *
 * Converts simple pointlist format to raw unsigned 16-bit PCM audio samples,
 * which can be used like this:
 * 	oscdraw MyPicture | aplay -f U16 -c 2 -r 4000 -
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <limits.h>
#include <string.h>
#include <inttypes.h>

typedef struct {
	float x, y;
} point;

int main(int argc, char **argv) {
	if (argc < 2) {
		printf("Usage: oscdraw <filename>\n");
		exit(0);
	}

	FILE *F = fopen(argv[1], "r");
	
	int i, n = 0, 
		alloced = 1000; // predict we have less than 1000 points
	
	point *p = malloc(sizeof(point) * alloced);
	point min = { 100000000, 1000000000 }, max = { 0, 0 }, t, d;

	n = 0;
	while (fscanf(F, "%f %f", &(t.x), &(t.y)) != EOF) {
		memcpy(p + n++, &t, sizeof(point));
		if (t.x < min.x) min.x = t.x;
		if (t.y < min.y) min.y = t.y;
		if (t.x > max.x) max.x = t.x;
		if (t.y > max.y) max.y = t.y;
		
		if (n >= alloced) {
			alloced += 1000;
			p = realloc(p, sizeof(point) * alloced);
		}
	}

	fclose(F);

	fprintf(stderr, "count %i\n", n);

	d.x = max.x - min.x;
	d.y = max.y - min.y;

	point *cur;
	
	fprintf(stderr, "size: %f %f\n", d.x, d.y);
	fprintf(stderr, "min: %f %f\n", min.x, min.y);
	fprintf(stderr, "max: %f %f\n", max.x, max.y);

	uint16_t tx, ty;
	i = 0;
	while (1) {
		cur = p + i;
		tx = (uint16_t) ((cur->x-min.x)/d.x*65535); 
		ty = (uint16_t) ((cur->y-min.y)/d.y*65535); 
		write(1, &tx, 2);
		write(1, &ty, 2);
		i = (i+1) % n;
	}

	free(p);
	return 0;
}
